import { RangeSet } from "@codemirror/state";
function rangeSetsEqual(a, b) {
  return RangeSet.eq([b], [a]);
}
function mapRangeSetToArray(rangeSet, callbackfn) {
  const values = new Array(rangeSet.size);
  if (values.length) {
    let valueIndex = 0;
    for (const rangeCursor = rangeSet.iter(); rangeCursor.value !== null; rangeCursor.next()) {
      values[valueIndex++] = callbackfn(rangeCursor.value, rangeCursor.from, rangeCursor.to);
    }
  }
  return values;
}
function reduceRangeSet(rangeSet, callbackfn, initialValue) {
  if (!rangeSet.size) {
    if (initialValue == null) {
      throw new TypeError("Reduce of empty RangeSet with no initial value");
    }
    return initialValue;
  }
  const rangeCursor = rangeSet.iter();
  let accumulator;
  if (initialValue == null) {
    accumulator = rangeCursor.value;
    rangeCursor.next();
  } else {
    accumulator = initialValue;
  }
  for (; rangeCursor.value !== null; rangeCursor.next()) {
    accumulator = callbackfn(accumulator, rangeCursor.value, rangeCursor.from, rangeCursor.to);
  }
  return accumulator;
}
const SetProxyUtils = /* @__PURE__ */ Object.freeze({
  add: (proxy, value) => {
    const set = proxy.unwrap();
    if (set.has(value)) {
      return proxy;
    }
    set.add(value);
    return createSetProxy(set);
  },
  delete: (proxy, value) => {
    const set = proxy.unwrap();
    if (!set.has(value)) {
      return proxy;
    }
    set.delete(value);
    return createSetProxy(set);
  },
  clear: (proxy) => {
    const set = proxy.unwrap();
    if (!set.size) {
      return proxy;
    }
    set.clear();
    return createSetProxy(set);
  },
  fp: /* @__PURE__ */ Object.freeze({
    add: (value) => (proxy) => {
      return SetProxyUtils.add(proxy, value);
    },
    delete: (value) => (proxy) => {
      return SetProxyUtils.delete(proxy, value);
    }
  })
});
function createSetProxy(set = /* @__PURE__ */ new Set()) {
  const proxy = {
    add: (value) => SetProxyUtils.add(proxy, value),
    addMany: (values) => values.reduce(SetProxyUtils.add, proxy),
    delete: (value) => SetProxyUtils.delete(proxy, value),
    deleteMany: (values) => values.reduce(SetProxyUtils.delete, proxy),
    clear: () => SetProxyUtils.clear(proxy),
    unwrap: () => set
  };
  Object.freeze(proxy);
  return proxy;
}
function mapStateEffectValue(effect, fn) {
  return fn(effect.value);
}
export {
  SetProxyUtils,
  createSetProxy,
  mapRangeSetToArray,
  mapStateEffectValue,
  rangeSetsEqual,
  reduceRangeSet
};
//# sourceMappingURL=index.js.map
